;+
; NAME:
;  kboplot
; PURPOSE:   (one line only)
;  Interactive plotting tool for the DES mySQL database contents.
; DESCRIPTION:
; CATEGORY:
;  Database
; CALLING SEQUENCE:
;  kboplot
; INPUTS:
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  Written by Marc W. Buie, Lowell Observatory, 2003 Jun 3
;-
pro kboplot_cleanup, tlb

   ; get pointer to state structure
   widget_control, tlb, get_uvalue=state


   free_lun,(*state).dblun

   ; Free up any dynamically allocated space with the state structure
   ptr_free,(*state).x
   ptr_free,(*state).y
   ptr_free,(*state).orbtype
   ptr_free,(*state).quality
   ptr_free,(*state).objectid

   ; Free up the state structure itself.
   ptr_free, state

end

pro kboplot_choosedata, state, axisname


   choice = picker((*state).choices[1,*],index=index)

   if choice eq '[[[CANCEL]]]' then return

   if axisname eq 'Abscissa' then begin
      (*state).xaxis = (*state).choices[0,index]
      (*state).xname = (*state).choices[1,index]
      (*state).xflip = (*state).choices[2,index]
   endif else begin
      (*state).yaxis = (*state).choices[0,index]
      (*state).yname = (*state).choices[1,index]
      (*state).yflip = (*state).choices[2,index]
   endelse

   (*state).dirty=1

end

pro kboplot_plot, state, hardcopy

   if (*state).dirty then begin

      cmd = ['select '+(*state).xaxis+','+(*state).yaxis+',', $
             'dclass.orbtype,dclass.quality,dclass.objectid', $
             'from dclass,elem,pos', $
             'where dclass.objectid=elem.objectid ', $
             '  and dclass.objectid=pos.objectid ', $
             (*state).qual_select+';']

      mysqlquery,(*state).dblun,cmd,x,y,orbtype,quality,objectid, $
         format='(f,f,a,i,a)'
      ptr_free,(*state).x
      ptr_free,(*state).y
      ptr_free,(*state).orbtype
      ptr_free,(*state).quality
      ptr_free,(*state).objectid
      (*state).x = ptr_new(x)
      (*state).y = ptr_new(y)
      (*state).orbtype = ptr_new(orbtype)
      (*state).quality = ptr_new(quality)
      (*state).objectid = ptr_new(objectid)
      (*state).dirty=0
   endif

   widget_control,(*state).drawwin, get_value=winnum
   wset,winnum

   if (*state).xflip eq 'norm' then $
      (*state).xr = minmax((*(*state).x)) $
   else $
      (*state).xr = maxmin((*(*state).x))

   if (*state).yflip eq 'norm' then $
      (*state).yr = minmax((*(*state).y)) $
   else $
      (*state).yr = maxmin((*(*state).y))

   plot,[0],[1],/nodata,xr=(*state).xr,yr=(*state).yr, $
      xtitle=(*state).xname,ytitle=(*state).yname

   oplot,(*(*state).x),(*(*state).y),psym=8,symsize=0.5

   ; bad error objects
   pos = strpos((*(*state).orbtype),'ERR2LARGE')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='707070'xl

   ; Resonant objects
   pos = strpos((*(*state).orbtype),':')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='0000ff'xl

   ; Scattered objects
   pos = strpos((*(*state).orbtype),'SCATNEAR')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='00ff00'xl

   ; Scattered objects
   pos = strpos((*(*state).orbtype),'SCATEXTD')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='ff00ff'xl

   ; Centaur objects
   pos = strpos((*(*state).orbtype),'CEN')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='00ffff'xl

   ; Classical objects
   pos = strpos((*(*state).orbtype),'CLASS')
   z=where(pos ge 0,count)
   if count ne 0 then $
      oplot,(*(*state).x)[z],(*(*state).y)[z], $
         psym=8,symsize=0.5,color='ffff00'xl

end

pro kboplot_postinfo, state, idx
   if idx lt 0 or idx ge n_elements( (*(*state).x) ) then return

   ; simple stuff
   str = 'Object name '+(*(*state).objectid)[idx]
   str = [str,'']
   newstr = (*state).xname+' = ' + string((*(*state).x)[idx])
   newstr = newstr + ', ' + (*state).yname+' = ' + string((*(*state).y)[idx])
   newstr = strcompress(newstr)
   str = [str,newstr]

   ; Current position information
   tab = string(byte(9))
   cmd = "select * from pos where objectid='"+(*(*state).objectid)[idx]+"';"
   mysqlcmd,(*state).dblun,cmd,answer,nlines
   heads = strsplit(answer[0],tab,/extract)
   words = strsplit(answer[1],tab,/extract)

   str = [str,'']

   z=where(heads eq 'jd')
   jdstr,double(words[z[0]]),0,jds
   newstr = 'Information determined for '+jds+' UT'
   z=where(heads eq 'hv')
   newstr = newstr+'       Hv='+words[z[0]]
   z=where(heads eq 'gv')
   newstr = newstr+', Gv='+words[z[0]]
   str = [str,newstr]
   str = [str,'']

   z=where(heads eq 'ra')
   rastr,double(words[z[0]]),1,ras
   newstr = '  RA = '+ras
   z=where(heads eq 'decl')
   decstr,double(words[z[0]]),0,decs
   newstr = newstr+', Dec = '+decs+'  J2000'
   z=where(heads eq 'vmag')
   newstr = newstr+'        V=' + $
      strcompress(string(float(words[z[0]]),format='(f10.1)'),/remove_all)
   str = [str,newstr]
   str = [str,'']

   z=where(heads eq 'sun')
   newstr = '  r=' + $
      strcompress(string(float(words[z[0]]),format='(f10.2)'),/remove_all)+ $
      ' AU'
   z=where(heads eq 'earth')
   newstr = newstr+',  delta=' + $
      strcompress(string(float(words[z[0]]),format='(f10.2)'),/remove_all)+ $
      ' AU'
   z=where(heads eq 'elong')
   newstr = newstr+',   Sel=' + $
      strcompress(string(float(words[z[0]]),format='(f10.1)'),/remove_all)+ $
      ' deg'
   z=where(heads eq 'phang')
   newstr = newstr+',   phase=' + $
      strcompress(string(float(words[z[0]]),format='(f10.1)'),/remove_all)+ $
      ' deg'
   str = [str,newstr]
   str = [str,'']

   z=where(heads eq 'x')
   newstr = '  x = ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=9)
   z=where(heads eq 'x_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)+ $
      ' AU'
   z=where(heads eq 'xv')
   newstr = newstr+'    xv = ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=11)
   z=where(heads eq 'xv_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=10)+ $
      ' AU/day'
   str = [str,newstr]

   z=where(heads eq 'y')
   newstr = '  y = ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=9)
   z=where(heads eq 'y_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)+ $
      ' AU'
   z=where(heads eq 'yv')
   newstr = newstr+'    yv = ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=11)
   z=where(heads eq 'yv_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=10)+ $
      ' AU/day'
   str = [str,newstr]

   z=where(heads eq 'z')
   newstr = '  z = ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=9)
   z=where(heads eq 'z_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)+ $
      ' AU'
   z=where(heads eq 'zv')
   newstr = newstr+'    zv = ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=11)
   z=where(heads eq 'zv_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(e15.4)',length=10)+ $
      ' AU/day'
   str = [str,newstr]
   str = [str,'']

   ; Orbital elements
   cmd = "select * from elem where objectid='"+(*(*state).objectid)[idx]+"';"
   mysqlcmd,(*state).dblun,cmd,answer,nlines
   heads = strsplit(answer[0],tab,/extract)
   words = strsplit(answer[1],tab,/extract)

   z=where(heads eq 'semi')
   newstr = '  a    = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)
   z=where(heads eq 'semi_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' AU '
   z=where(heads eq 'manom')
   newstr = newstr+'    M    = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)
   z=where(heads eq 'manom_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' deg'
   str = [str,newstr]

   z=where(heads eq 'ecc')
   newstr = '  ecc  = ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)
   z=where(heads eq 'ecc_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)+'    '
   z=where(heads eq 'arg')
   newstr = newstr+'    arg  = ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)
   z=where(heads eq 'arg_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.4)',length=6)+' deg'
   str = [str,newstr]

   z=where(heads eq 'inc')
   newstr = '  inc  = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)
   z=where(heads eq 'inc_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' deg'
   z=where(heads eq 'node')
   newstr = newstr+'    node = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)
   z=where(heads eq 'node_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' deg'
   str = [str,newstr]
   str = [str,'']

   z=where(heads eq 'peri')
   newstr = '    q  = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=7)
   z=where(heads eq 'peri_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' AU'
   z=where(heads eq 'period')
   newstr = newstr+'      P  = ' + $
      strn(float(words[z[0]]),format='(f10.1)',length=8)
   z=where(heads eq 'period_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.1)',length=6)+' years'
   str = [str,newstr]

   z=where(heads eq 'aphelion')
   newstr = '    Q  = ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=7)
   z=where(heads eq 'aphelion_s')
   newstr = newstr+' +- ' + $
      strn(float(words[z[0]]),format='(f10.2)',length=6)+' AU'
   z=where(heads eq 'epoch')
   jdstr,double(words[z[0]]),0,jds
   newstr = newstr+'      Epoch '+jds
   str = [str,newstr]
   str = [str,'']

   z=where(heads eq 'erra')
   newstr = ' Positional uncertainty  erra = ' + $
      strn(float(words[z[0]]),format='(f12.1)')
   z=where(heads eq 'errb')
   newstr = newstr+',  errb = ' + $
      strn(float(words[z[0]]),format='(f12.1)')+' arcsec'
   str = [str,newstr]

   z=where(heads eq 'lastobs')
   jdlast=double(words[z[0]])
   jdstr,jdlast,100,jds
   newstr = ' Last observed ' + jds
   curjd = systime(/julian,/utc)
   diff = curjd - jdlast
   if diff lt 270 then begin
      newstr = newstr+' (' + $
         strn(fix(diff))+' days ago)'
   endif else begin
      newstr = newstr+' (' + $
         strn(float(diff)/365.25,format='(f10.1)')+' years ago)'
   endelse
   z=where(heads eq 'arc')
   arc=float(words[z[0]])
   if arc lt 45 then begin
      newstr = newstr+'  '+ $
         strn(float(arc),format='(f10.1)')+' day arc'
   endif else if arc lt 270 then begin
      newstr = newstr+'  '+ $
         strn(float(arc)/30.0,format='(f10.1)')+' month arc'
   endif else begin
      newstr = newstr+'  '+ $
         strn(float(arc)/365.25,format='(f10.1)')+' year arc'
   endelse
   z=where(heads eq 'nobs')
   newstr = newstr + ', '+strn(long(words[z[0]]))+' observations'
   str = [str,newstr]
   str = [str,'']

   ; Orbital elements
   cmd = "select * from dclass where objectid='"+(*(*state).objectid)[idx]+"';"
   mysqlcmd,(*state).dblun,cmd,answer,nlines
   heads = strsplit(answer[0],tab,/extract)
   words = strsplit(answer[1],tab,/extract)

   z=where(heads eq 'quality')
   newstr = 'Quality flag = '+strn(long(words[z[0]]))
   str = [str,newstr]
   str = [str,'']

   fmt='(a,3(1x,a9))'
   labl='Type:     '
   z1=where(heads eq 'orbtype')
   z2=where(heads eq 'orbtype_p')
   z3=where(heads eq 'orbtype_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='axisobj   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='ecceobj   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='qmin      '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='amean     '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='amin      '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='emean     '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='emax      '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='imean     '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='fracstop  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='cj        '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   str = [str,'']

   labl='libcent0  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libamp0   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libcent1  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libamp1   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libcent2  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libamp2   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libcent3  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libamp3   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libcent4  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='libamp4   '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   str = [str,'']

   labl='kozaimean '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]
   labl='kozaiamp  '
   z1=where(heads eq strtrim(labl,2))
   z2=where(heads eq strtrim(labl,2)+'_p')
   z3=where(heads eq strtrim(labl,2)+'_m')
   newstr = string(labl,words[z1[0]],words[z2[0]],words[z3[0]],format=fmt)
   str = [str,newstr]

   widget_control, (*state).infoid, set_value=str
end

pro kboplot_eve, event

   widget_control, event.top, GET_UVALUE=state

   if event.id eq (*state).mainbase then $
      event_name = 'Mainbase' $
   else $
      widget_control, event.id,  GET_UVALUE=event_name, /HOURGLASS

   exit = event_name eq 'THE_MENU'
   if exit then exit = event.value eq 'Exit'

   case event_name of

      'THE_MENU': begin
         case event.value of

            'Absicca': begin
               kboplot_choosedata, state, 'Abscissa'
            end

            'Ordinate': begin
               kboplot_choosedata, state, 'Ordinate'
            end

         ; Quality submenu
            'Q=3': begin
               (*state).qual_select = 'and dclass.quality=3 ';
               (*state).dirty=1
            end

            'Q>1': begin
               (*state).qual_select = 'and dclass.quality>1 ';
               (*state).dirty=1
            end

            'Q>0': begin
               (*state).qual_select = 'and dclass.quality>0 ';
               (*state).dirty=1
            end

            'Q=2': begin
               (*state).qual_select = 'and dclass.quality=2 ';
               (*state).dirty=1
            end

            'Q=1': begin
               (*state).qual_select = 'and dclass.quality=1 ';
               (*state).dirty=1
            end

            'Q=0': begin
               (*state).qual_select = 'and dclass.quality=0 ';
               (*state).dirty=1
            end

            'Show All': begin
               (*state).qual_select = '';
               (*state).dirty=1
            end

            'Exit' : begin
               widget_control, event.top, /DESTROY
               RETURN
            end

            else: begin
               message, 'Unknown menu event:', /INFO
               help, event, /STRUCTURE
            end

         endcase

         kboplot_plot, state, 0

      end ; THE_MENU

      'Mainbase': begin

         ; Use if you have other widgets on screen, need to take off their
         ;   size from the event x,y size.
;         info=widget_info((*state).colbaseid,/geometry)
;         widget_control,(*state).drawwin,xsize=event.x,ysize=event.y-info.ysize

         ; Use if draw window is only thing in the tool.
         widget_control,(*state).drawwin,xsize=event.x,ysize=event.y
;         widget_control,(*state).drawwin,xsize=event.x,ysize=event.y-info.ysize

;        refresh window here
         kboplot_plot, state, 0
      end

      'Window': begin
         if event.type le 1 then begin
            if event.press then begin
               (*state).xpress = event.x
               (*state).ypress = event.y
            endif else begin
               if event.x ne (*state).xpress and event.y ne (*state).ypress then begin
                  print,'box at (x1,y1,x2,y2) ',(*state).xpress,(*state).ypress,event.x,event.y
               endif else begin
                  res=convert_coord(event.x,event.y,/device,/to_data)
                  diff = sqrt( ((*(*state).x) - res[0])^2 + $
                               ((*(*state).y) - res[1])^2 ) 
                  idx = where(diff eq min(diff))
                  kboplot_postinfo,state,idx[0]
               endelse
            endelse
         endif else if event.type eq 5 then begin
            print,'ASCII: ',event.ch
         endif else if event.type eq 6 then begin
            print,'non-ASCII: ',event.key
         endif else begin
            help, event, /STRUCTURE
         endelse
      end

      else: begin
         print,'EVENT NAME: ',event_name
         message, 'Unknown event:', /INFO
         help, event, /STRUCTURE
      end

   endcase

end ; end of event handler

pro kboplot

   ; optional
   if xregistered('kboplot') then return

   if (!d.flags and 256) eq 0 then begin
      print, 'Error. No windowing device. KBOPLOT cannot be started.'
      return
   endif

   ;Define the main base.
   mainbase = widget_base( TITLE='KBOPLOT: KBO database plotter', $
                           /COLUMN, UVALUE=0, MBAR=bar, /TLB_SIZE_EVENTS )

   menu = CW_PdMenu(bar, /RETURN_NAME, $
                    ['1\File',$
                     '0\action 1',$
                     '0\action 2',$
                     '2\Exit',$
                     '1\Setup',$
                     '0\Absicca',$
                     '2\Ordinate',$
                     '1\Quality',$
                     '0\Q=3', $
                     '0\Q>1', $
                     '0\Q>0', $
                     '0\Q=2', $
                     '0\Q=1', $
                     '0\Q=0', $
                     '2\Show All', $
                     '1\Tools',$
                     '0\tool 1', $
                     '0\tool 2', $
                     '2\tool 3'], UVALUE='THE_MENU', /MBAR)

   base = widget_base(mainbase)

   win1 = widget_draw( base, XSIZE=400, YSIZE=400, RETAIN=2, $
                       /BUTTON_EVENTS, UVALUE='Window', KEYBOARD_EVENTS=2 )

   openmysql,dblun,'des'

   choices = [['elem.semi', 'semi-major axis',      'norm'], $
              ['elem.ecc',  'eccentricity',         'norm'], $
              ['elem.inc',  'inclination',          'norm'], $
              ['elem.peri', 'Perihelion Distance',  'norm'], $
              ['elem.aphelion', 'Aphelion Distance', 'norm'], $
              ['elem.period', 'Period',             'norm'], $
              ['elem.erra', 'semi-major axis of error ellipse', 'norm'], $
              ['elem.errb', 'semi-minor axis of error ellipse', 'norm'], $
              ['elem.arc',  'Astrometric arc (days)', 'norm'], $
              ['elem.nobs', 'Number of observations', 'norm'], $
              ['pos.vmag',  'Vmag',                 'flip'], $
              ['pos.hv',    'HV',                   'flip'], $
              ['pos.sun',   'Heliocentric Distance','norm'], $
              ['pos.earth', 'Geocentric Distance',  'norm'], $
              ['pos.elong', 'Solar Elongation',     'norm'], $
              ['pos.ra',    'Right Ascension (J2000)', 'norm'], $
              ['pos.dec',   'Declination (J2000)',  'norm'], $
              ['pos.x',     'X',                    'norm'], $
              ['pos.y',     'Y',                    'norm'], $
              ['pos.z',     'Z',                    'norm'], $
              ['pos.xv',    'X velocity',           'norm'], $
              ['pos.yv',    'Y velocity',           'norm'], $
              ['pos.zv',    'Z velocity',           'norm'] ]

   infobase = widget_base(group_leader=mainbase,/column, $
                           title='Object Information Window')

   infoid   = widget_text(infobase, xsize=80, ysize=43, /scroll)


   state = ptr_new({ $

      ; Data and information in the widget
      choices: choices, $
      dblun: dblun, $
      dirty: 1, $                ; if set, need to rerun query
      qual_select: '', $         ; Orbit quality select string
      xpress: -1L, $
      xr: [0.,1.], $
      ypress: -1L, $
      yr: [0.,1.], $

      x:     ptr_new(), $
      xaxis: choices[0,0], $
      xname: choices[1,0], $
      xflip: choices[2,0], $
      y:     ptr_new(), $
      yaxis: choices[0,1], $
      yname: choices[1,1], $
      yflip: choices[2,1], $
      objectid: ptr_new(), $
      orbtype: ptr_new(), $
      quality: ptr_new(), $

      ; Widget ids
      drawwin: win1, $           ; ID of main draw window
      infoid:  infoid, $         ; ID of information window

      mainbase: mainbase $       ; ID of top level base.

      })

   ;Stash the state structure pointer.
   widget_control, mainbase, SET_UVALUE=state

   ;Realize the main base.
   widget_control, mainbase, /REALIZE

   ;Realize the info base.
   widget_control, infobase, /REALIZE

   ;Generate the initial default plot
   kboplot_plot, state, 0

   ; Give control to the XMANAGER.
   xmanager, 'kboplot', mainbase, $
             EVENT_HANDLER='kboplot_eve',/NO_BLOCK, $
             GROUP_LEADER=mainbase, CLEANUP='kboplot_cleanup'

end
