;+
; NAME:
;  obsdur
; PURPOSE:
;  Calculate estimate of total duration of an exposure with PCCD.
; DESCRIPTION:
; CATEGORY:
;  Data Acquisition
; CALLING SEQUENCE:
;  ans=obsdur(filter,exptime,colormode,npat)
; INPUTS:
;  filter    - Filter for observation (or anchor filter for color mode).
;  exptime   - Exposure time for indicated filter (in seconds).
;  colormode - Character, 'c' means use color sequence, anything else means
;                just this filter.
;  npat      - Number of exposures (or color patterns).
; OPTIONAL INPUT PARAMETERS:
; KEYWORD INPUT PARAMETERS:
; OUTPUTS:
;  return value is the estimated time to complete the observation (in seconds).
; KEYWORD OUTPUT PARAMETERS:
; COMMON BLOCKS:
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
; MODIFICATION HISTORY:
;  Written by Marc W. Buie, Lowell Observatory, 2002 Jan 27
;-
function obsdur,filter,exptime,colormode,npat

   ; This table needs to be kept in sync with the PCCD daemon on hermitr
   ;       0    1    2    3    4    5    6    7    8     9
   ;      Open  B    V    R    w    x    p    I    M2    M    
   dwell=[0.6, 4.8, 1.2, 1.0, 1.3, 1.5, 2.5, 1.2, 3.12, 3.12]

   ; Overhead per exposure, this includes 8 sec for the readout and 2 sec
   ;  for the exposure time syncronization and saving to disk.
   overhead = 10.0  ; seconds

   if colormode eq 'c' then begin
      case filter of
         1: begin
            idx=[2,1,3]
            endif
         2: begin
            idx=[2,1]
            endif
         3: begin
            idx=[3,2]
            endif
         4: begin
            idx=[2,1,4]
            endif
         5: begin
            idx=[2,1,4,2,5]
            endif
         6: begin
            idx=[2,1,4,2,5,6]
            endif
         7: begin
            idx=[3,1,2,7]
            endif
         8: begin
            idx=[8,9]
            endif
         9: begin
            idx=[2,1,9]
            endif
         else: begin
            idx=filter
            endif
      endcase
      dur = npat * (exptime*dwell[idx]/dwell[filter] + overhead) + $
               exptime*dwell[idx[0]]/dwell[filter] + overhead
   endif else begin
      dur = npat * (exptime + overhead)
   endelse

end
